! ###########################################################
!                 SDSU v2 LUT creation routines
!                       - mktab_vir -
!
! This subroutine creates the Mie lookup table for the
! microwave radiometer simulator.
!
!                              Created by H. M. (May 2009)
!
! Since SDSU v2.1.0, loops over "npts_ncnc" has been updated
! so a two moment scheme can be readily implemented.
!                                         H. M. (Aug 2009)
!
 Subroutine mktab_vir
   Use LUTparameters
   Use MCRPparameters
   Use VIRparameters, only : knang,nang,ang
   Implicit NONE
!
! Local variables
!
   real :: wavelngth
   integer :: nspec,i,j,nwvln,k
   integer :: npts_ncnc0
   character*50  :: fname_out(0:6),fname_lab(0:6)
   character*100 :: pathandfile
   character*15   :: frmt
   character*4, external :: mklab_lut_vir
   character*4, dimension(:), allocatable :: fnametag_visir
   character*2 :: i2a
!
   real(8) :: wld,tmpd,lwcd,iwcd,Ncd,cextd,cscad
   real(8), dimension(knang) :: phsfd
   real, dimension(:,:,:),   allocatable :: cext_lut,csca_lut
   real, dimension(:,:,:,:), allocatable :: phsf_lut
!
   write(*,*) '- Visile/infrared imager -'
!
!  Wavelength name tags for the output lookup tables
!  defined by "src/simulator/hydromet/mklab_lut.f90".
!
   allocate ( fnametag_visir(nwvlnVI) )
   do nwvln = 1, nwvlnVI
      fnametag_visir(nwvln) = mklab_lut_vir ( wvln_visir(nwvln) )
   end do
!
   allocate ( cext_lut(      npts_hydro,npts_temp,npts_ncnc) )
   allocate ( csca_lut(      npts_hydro,npts_temp,npts_ncnc) )
   allocate ( phsf_lut(knang,npts_hydro,npts_temp,npts_ncnc) )
!
! Info files
!
   fname_lab(0) ='melt-p/'//snow%functype//trim(snow%name)//'_' & 
                          //rain%functype//trim(rain%name)//'-vir'
   fname_lab(1) = cldi%functype//'/'//trim(cldi%name)//'-vir'
   fname_lab(2) = cldw%functype//'/'//trim(cldw%name)//'-vir'
   fname_lab(3) = grpl%functype//'/'//trim(grpl%name)//'-vir'
   fname_lab(4) = hail%functype//'/'//trim(hail%name)//'-vir'
   fname_lab(5) = snow%functype//'/'//trim(snow%name)//'-vir'
   fname_lab(6) = rain%functype//'/'//trim(rain%name)//'-vir'
!
   pathandfile = dir_out//trim(fname_lab(1))//'.info'
   open ( unit=15, file=pathandfile )
   write(15,*) ' - Cloud ice: ',trim(cldi%name)//'('//cldi%functype//')'
   Do i = 1, cldi%nparams
      write(15,10) '  PSD parameter #',i,'=',cldi%paramset(i)
   End do
!
   pathandfile = dir_out//trim(fname_lab(2))//'.info'
   open ( unit=15, file=pathandfile )
   write(15,*) ' - Cloud water: ',trim(cldw%name)//'('//cldw%functype//')'
   Do i = 1, cldw%nparams
      write(15,10) '  PSD parameter #',i,'=',cldw%paramset(i)
   End do
!
   pathandfile = dir_out//trim(fname_lab(3))//'.info'
   open ( unit=15, file=pathandfile )
   write(15,*) ' - Graupel: ',trim(grpl%name)//'('//grpl%functype//')'
   Do i = 1, grpl%nparams
      write(15,10) '  PSD parameter #',i,'=',grpl%paramset(i)
   End do
!
   pathandfile = dir_out//trim(fname_lab(4))//'.info'
   open ( unit=15, file=pathandfile )
   write(15,*) ' - Hail: ',trim(hail%name)//'('//hail%functype//')'
   Do i = 1, hail%nparams
      write(15,10) '  PSD parameter #',i,'=',hail%paramset(i)
   End do
!
   pathandfile = dir_out//trim(fname_lab(5))//'.info'
   open ( unit=15, file=pathandfile )
   write(15,*) ' - Snow: ',trim(snow%name)//'('//snow%functype//')'
   Do i = 1, snow%nparams
      write(15,10) '  PSD parameter #',i,'=',snow%paramset(i)
   End do
!
   pathandfile = dir_out//trim(fname_lab(6))//'.info'
   open ( unit=15, file=pathandfile )
   write(15,*) ' - Rain: ',trim(rain%name)//'('//rain%functype//')'
   Do i = 1, rain%nparams
      write(15,10) '  PSD parameter #',i,'=',rain%paramset(i)
   End do
!
10 format(a,i2,a,1P1e10.3)
!
! Create LUTs
!
   do nwvln = 1 , nwvlnVI
!
      write(*,*) 'Wavelength = ',wvln_visir(nwvln)
!
      do nspec = 1, 6
         fname_out(nspec) = trim(fname_lab(nspec))// &
                            fnametag_visir(nwvln)//'.dat' 
      end do
!
      wavelngth = wvln_visir(nwvln) ; wld = dble(wavelngth)
      frmt = '(1P'//i2a(2+nang,2)//'E14.6)'
!
! cloud ice
!
      nspec = 1
!
      if ( cldi%twomomnt ) then
         npts_ncnc0 = npts_ncnc
      else
         npts_ncnc0 = 1
      endif
!
      pathandfile = dir_out//trim(fname_out(nspec))
      open(2,file=pathandfile)
      write(*,*) 'Cloud ice -'
!
!       Write header for output file
!
      write(2,*) npts_temp, npts_hydro, npts_ncnc0
      write(2,92) (pts_temp_i(i),i=1,npts_temp)
      write(2,92) (pts_hydro(j),j=1,npts_hydro)
      write(2,92) (pts_ncncc(i),i=1,npts_ncnc0)
      write(2,91) wavelngth
!
      do k = 1, npts_ncnc0
         Ncd = dble(pts_ncncc(k))
         do i = 1, npts_temp
            tmpd = dble(pts_temp_i(i))
!
            If ( cldi%indpndtT .and. i > 1 ) then
               cext_lut(:,i,k) = cext_lut(:,1,k)
               csca_lut(:,i,k) = csca_lut(:,1,k)
               phsf_lut(1:nang,:,i,k) = phsf_lut(1:nang,:,1,k)
               cycle
            Endif
!               
            do j = 1, npts_hydro
               iwcd = dble(pts_hydro(j))
               Call mie_cldi_vir(wld,tmpd,iwcd,Ncd,cextd,cscad,phsfd)
               cext_lut(j,i,k) = cextd
               csca_lut(j,i,k) = cscad
               phsf_lut(1:nang,j,i,k) = phsfd(1:nang)
            end do
         end do
      end do
      write(2,frmt) (((cext_lut(j,i,k),csca_lut(j,i,k),phsf_lut(1:nang,j,i,k), &
                       j=1,npts_hydro),i=1,npts_temp),k=1,npts_ncnc0)
      close (2)
!
! cloud water
!
      nspec = 2
!
      if ( cldw%twomomnt ) then
         npts_ncnc0 = npts_ncnc
      else
         npts_ncnc0 = 1
      endif
!
      pathandfile = dir_out//trim(fname_out(nspec))
      open(2,file=pathandfile)
      write(*,*) 'Cloud water -'
!
!       Write header for output file
!
      write(2,*) npts_temp, npts_hydro, npts_ncnc0
      write(2,92) (pts_temp_w(i),i=1,npts_temp)
      write(2,92) (pts_hydro(j),j=1,npts_hydro)
      write(2,92) (pts_ncncc(i),i=1,npts_ncnc0)
      write(2,91) wavelngth
!
      do k = 1, npts_ncnc0
         Ncd = dble(pts_ncncc(k))
         do i = 1, npts_temp
            tmpd = dble(pts_temp_w(i))
!
            If ( cldw%indpndtT .and. i > 1 ) then
               cext_lut(:,i,k) = cext_lut(:,1,k)
               csca_lut(:,i,k) = csca_lut(:,1,k)
               phsf_lut(1:nang,:,i,k) = phsf_lut(1:nang,:,1,k)
               cycle
            Endif
!               
            do j = 1, npts_hydro
               lwcd = dble(pts_hydro(j))
               Call mie_cldw_vir(wld,tmpd,lwcd,Ncd,cextd,cscad,phsfd)
               cext_lut(j,i,k) = cextd
               csca_lut(j,i,k) = cscad
               phsf_lut(1:nang,j,i,k) = phsfd(1:nang)
            end do
         end do
      end do
      write(2,frmt) (((cext_lut(j,i,k),csca_lut(j,i,k),phsf_lut(1:nang,j,i,k), &
                       j=1,npts_hydro),i=1,npts_temp),k=1,npts_ncnc0)
      close (2)
!
! graupel
!
      nspec = 3
!
      if ( grpl%twomomnt ) then
         npts_ncnc0 = npts_ncnc
      else
         npts_ncnc0 = 1
      endif
!
      pathandfile = dir_out//trim(fname_out(nspec))
      open(2,file=pathandfile)
      write(*,*) 'Graupel -'
!
!       Write header for output file
!
      write(2,*) npts_temp, npts_hydro, npts_ncnc0
      write(2,92) (pts_temp_i(i),i=1,npts_temp)
      write(2,92) (pts_hydro(j),j=1,npts_hydro)
      write(2,92) (pts_ncncp(i),i=1,npts_ncnc0)
      write(2,91) wavelngth
!
      do k = 1, npts_ncnc0
         Ncd = dble(pts_ncncp(k))
         do i = 1, npts_temp
            tmpd = dble(pts_temp_i(i))
!
            If ( grpl%indpndtT .and. i > 1 ) then
               cext_lut(:,i,k) = cext_lut(:,1,k)
               csca_lut(:,i,k) = csca_lut(:,1,k)
               phsf_lut(1:nang,:,i,k) = phsf_lut(1:nang,:,1,k)
               cycle
            Endif
!               
            do j = 1, npts_hydro
               iwcd = dble(pts_hydro(j))
               Call mie_grpl_vir(wld,tmpd,iwcd,Ncd,cextd,cscad,phsfd)
               cext_lut(j,i,k) = cextd
               csca_lut(j,i,k) = cscad
               phsf_lut(1:nang,j,i,k) = phsfd(1:nang)
            end do
         end do
      end do
      write(2,frmt) (((cext_lut(j,i,k),csca_lut(j,i,k),phsf_lut(1:nang,j,i,k), &
                       j=1,npts_hydro),i=1,npts_temp),k=1,npts_ncnc0)
      close (2)
!
! hail
!
      nspec = 4
!
      if ( hail%twomomnt ) then
         npts_ncnc0 = npts_ncnc
      else
         npts_ncnc0 = 1
      endif
!
      pathandfile = dir_out//trim(fname_out(nspec))
      open(2,file=pathandfile)
      write(*,*) 'Hail -'
!
!       Write header for output file
!
      write(2,*) npts_temp, npts_hydro, npts_ncnc0
      write(2,92) (pts_temp_i(i),i=1,npts_temp)
      write(2,92) (pts_hydro(j),j=1,npts_hydro)
      write(2,92) (pts_ncncp(i),i=1,npts_ncnc0)
      write(2,91) wavelngth
!
      do k = 1, npts_ncnc0
         Ncd = dble(pts_ncncp(k))
         do i = 1, npts_temp
            tmpd = dble(pts_temp_i(i))
!
            If ( hail%indpndtT .and. i > 1 ) then
               cext_lut(:,i,k) = cext_lut(:,1,k)
               csca_lut(:,i,k) = csca_lut(:,1,k)
               phsf_lut(1:nang,:,i,k) = phsf_lut(1:nang,:,1,k)
               cycle
            Endif
!               
            do j = 1, npts_hydro
               iwcd = dble(pts_hydro(j))
               Call mie_hail_vir(wld,tmpd,iwcd,Ncd,cextd,cscad,phsfd)
               cext_lut(j,i,k) = cextd
               csca_lut(j,i,k) = cscad
               phsf_lut(1:nang,j,i,k) = phsfd(1:nang)
            end do
         end do
      end do
      write(2,frmt) (((cext_lut(j,i,k),csca_lut(j,i,k),phsf_lut(1:nang,j,i,k), &
                       j=1,npts_hydro),i=1,npts_temp),k=1,npts_ncnc0)
      close (2)
!
! snow
!
      nspec = 5
!
      if ( snow%twomomnt ) then
         npts_ncnc0 = npts_ncnc
      else
         npts_ncnc0 = 1
      endif
!
      pathandfile = dir_out//trim(fname_out(nspec))
      open(2,file=pathandfile)
      write(*,*) 'Snow -'
!
!       Write header for output file
!
      write(2,*) npts_temp, npts_hydro, npts_ncnc0
      write(2,92) (pts_temp_i(i),i=1,npts_temp)
      write(2,92) (pts_hydro(j),j=1,npts_hydro)
      write(2,92) (pts_ncncp(i),i=1,npts_ncnc0)
      write(2,91) wavelngth
!
      do k = 1, npts_ncnc0
         Ncd = dble(pts_ncncp(k))
         do i = 1, npts_temp
            tmpd = dble(pts_temp_i(i))
!
            If ( snow%indpndtT .and. i > 1 ) then
               cext_lut(:,i,k) = cext_lut(:,1,k)
               csca_lut(:,i,k) = csca_lut(:,1,k)
               phsf_lut(1:nang,:,i,k) = phsf_lut(1:nang,:,1,k)
               cycle
            Endif
!               
            do j = 1, npts_hydro
               iwcd = dble(pts_hydro(j))
               Call mie_snow_vir(wld,tmpd,iwcd,Ncd,cextd,cscad,phsfd)
               cext_lut(j,i,k) = cextd
               csca_lut(j,i,k) = cscad
               phsf_lut(1:nang,j,i,k) = phsfd(1:nang)
            end do
         end do
      end do
      write(2,frmt) (((cext_lut(j,i,k),csca_lut(j,i,k),phsf_lut(1:nang,j,i,k), &
                       j=1,npts_hydro),i=1,npts_temp),k=1,npts_ncnc0)
      close (2)
!
! Rain water
!
      nspec = 6
!
      if ( rain%twomomnt ) then
         npts_ncnc0 = npts_ncnc
      else
         npts_ncnc0 = 1
      endif
!
      pathandfile = dir_out//trim(fname_out(nspec))
      open(2,file=pathandfile)
      write(*,*) 'Rain -'
!
!       Write header for output file
!
      write(2,*) npts_temp, npts_hydro, npts_ncnc0
      write(2,92) (pts_temp_w(i),i=1,npts_temp)
      write(2,92) (pts_hydro(j),j=1,npts_hydro)
      write(2,92) (pts_ncncp(i),i=1,npts_ncnc0)
      write(2,91) wavelngth
!
      do k = 1, npts_ncnc0
         Ncd = dble(pts_ncncp(k))
         do i = 1, npts_temp
            tmpd = dble(pts_temp_w(i))
!
            If ( rain%indpndtT .and. i > 1 ) then
               cext_lut(:,i,k) = cext_lut(:,1,k)
               csca_lut(:,i,k) = csca_lut(:,1,k)
               phsf_lut(1:nang,:,i,k) = phsf_lut(1:nang,:,1,k)
               cycle
            Endif
!               
            do j = 1, npts_hydro
               lwcd = dble(pts_hydro(j))
               Call mie_rain_vir(wld,tmpd,lwcd,Ncd,cextd,cscad,phsfd)
               cext_lut(j,i,k) = cextd
               csca_lut(j,i,k) = cscad
               phsf_lut(1:nang,j,i,k) = phsfd(1:nang)
            end do
         end do
      end do
      write(2,frmt) (((cext_lut(j,i,k),csca_lut(j,i,k),phsf_lut(1:nang,j,i,k), &
                       j=1,npts_hydro),i=1,npts_temp),k=1,npts_ncnc0)
      close (2)
!
! Output formatting
!
91 Format ( 1F12.5 )     ! wavelength
92 Format ( 1P50E13.5 )  ! LUT vectors
95 Format ( 3I5 )        ! LUT dimension sizes
!
   end do
!
   deallocate ( cext_lut, csca_lut, phsf_lut )
   Return
 end Subroutine mktab_vir
