! ###########################################################
!                 SDSU v2 LUT creation routines
!                       - paramLUT_set -
!
! This subroutine sets up LUT parameters.
! 
!                                Created by H. M. (May, 2009)
!
 Subroutine paramLUT_set
   Use LUTparameters
!
! ** LUT dimensions
!
! npts_hydro: dimension size for hydrometer water content
!
   npts_hydro = 37 ; allocate( pts_hydro(npts_hydro) )
!
! pts_hydro: hydrometeor water content vector in [g/m^3]
!
   pts_hydro = (/ 0.001, 0.002, 0.003, 0.004, 0.005, 0.006, &
                  0.007, 0.008, 0.009, 0.01, 0.02, 0.03, 0.04, &
                  0.05, 0.06, 0.07, 0.08, 0.09, 0.1, 0.2, 0.3, &
                  0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0, 2.0, 3.0, &
                  4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0 /)
!
! npts_temp: dimension size for temperature
!
   npts_temp = 12 ; allocate( pts_temp_w(npts_temp),pts_temp_i(npts_temp) )
!
! pts_temp_w: temperature vector in [K] for liquid hydrometeor species
!             (rain and cloud water).
!
   pts_temp_w = (/ 255., 260., 265., 270., 275., 280., &
                   285., 290., 295., 300., 305., 310. /)
!
! pts_temp_i: temperature vector in [K] for frozen hydrometeor species
!             (cloud ice, snow, graupel, and hail).
!
   pts_temp_i = (/ 180., 190., 200., 210., 220., 230., &
                   240., 250., 260., 270., 275., 280. /)
!
! / Updated for SDSU v2.1.0 /
! npts_ncnc: dimension size for hydrometer number concentration
!            Used only for two-moment microphysical schemes
!
   npts_ncnc = 30; allocate( pts_ncncc(npts_ncnc),pts_ncncp(npts_ncnc) )
!
! pts_ncncc: hydrometeor number concentration vector in [1/m^3] for cloud
!            particles. Used only for two-moment microphysical schemes.
!
   pts_ncncc = (/ 1.e+3, 3.e+3, 1.e+4, 2.e+4, 4.e+4, 6.e+4, 8.e+4, 1.e+5, &
                  2.e+5, 4.e+5, 6.e+5, 8.e+5, 1.e+6, 2.e+6, 4.e+6, 6.e+6, &
                  8.e+6, 1.e+7, 2.e+7, 4.e+7, 6.e+7, 8.e+7, 1.e+8, 2.e+8, &
                  4.e+8, 6.e+8, 8.e+8, 1.e+9, 3.e+9, 1.e+10 /)
!
! pts_ncncp: hydrometeor number concentration vector in [1/m^3] for precip
!            particles. Used only for two-moment microphysical schemes.
!
   pts_ncncp = (/ 1.e+0, 3.e+0, 1.e+1, 2.e+1, 4.e+1, 6.e+1, 8.e+1, 1.e+2, &
                  2.e+2, 4.e+2, 6.e+2, 8.e+2, 1.e+3, 2.e+3, 4.e+3, 6.e+3, &
                  8.e+3, 1.e+4, 2.e+4, 4.e+4, 6.e+4, 8.e+4, 1.e+5, 2.e+5, &
                  4.e+5, 6.e+5, 8.e+5, 1.e+6, 3.e+6, 1.e+7 /)
!
! //////////////// Microwave Radiometer /////////////////
!
! Mie LUTs are constructed for MW radiometer simulation when microw=.true.
!
   microw = .true.
!
! ** Sensor channel specs
!
! nfreqMW: number of microwave radiometer channel frequencies
!
   nfreqMW = 5; allocate( freq_rdmtr(nfreqMW) )
!
! MW radiometer channel frequencies in [GHz]
!
   freq_rdmtr = (/ 10.7, 19.35, 21.3, 37.0, 85.5 /)
!
! ////////////////        Radar         /////////////////
!
! Mie LUTs are constructed for radar simulation when radar=.true.
!
   radar = .true. 
!
! ** Sensor channel specs
!
! nfreqRD: number of radar channel frequencies
!
   nfreqRD = 2; allocate( freq_radar(nfreqRD) )
!
! Radar channel frequencies in [GHz]
!
   freq_radar = (/ 13.8, 35.5 /)
!
! ////////////////  Visible/IR Imager  /////////////////
!
! Mie LUTs are constructed for visible/IR simulation when visIR=.true.
!
   visIR = .true. 
!
! ** Sensor channel specs
!
! nwvlnVI: number of visible/IR channel wavelengths
!
   nwvlnVI = 5; allocate( wvln_visir(nwvlnVI) )
!
! Visible/infrared channel wavelengths in [micron]
!
   wvln_visir = (/ 0.62, 1.6, 3.8, 10.8, 12.0 /)
!
 End Subroutine paramLUT_set
